#!/bin/sh

# luna firmware upgrade  script
# $1 image destination (0 or 1) 
# Kernel and root file system images are assumed to be located at the same directory named uImage and rootfs respectively
# ToDo: use arugements to refer to kernel/rootfs location.

k_img="uImage"
r_img="rootfs"
img_ver="fwu_ver"
img_sh="fwu.sh"
md5_cmp="md5.txt"
md5_cmd="/bin/md5sum"
#md5 run-time result
md5_tmp="md5_tmp" 
md5_rt_result="md5_rt_result.txt"

if 
        cat /proc/mtd | grep k1
then    
        kname="k"$1""
        rname="r"$1""
elif      
#it's v2801rw check if 16M flash
        cat /proc/mtd | grep linux
then            
        kname="linux"
        rname="rootfs"
else
#it's 8M flash
        kname="k0"
        rname="r0"
fi
echo "==>$kname  ==>$rname"
#add by runt for bug#0001005 on 2019-04-10
if 
	cat /var/config/lastgood.xml | grep 'Value Name="ONUTYPE_CHECK_SWITCH" Value="1"'
then
	###################ONU DEVICE MODEL VERTIFICATION####################
	echo ONU DEVICE MODEL VERTIFICATION
	tar -xf $2 $img_ver 
	upgradeModel1=$(sed -n 2p $img_ver)
	upgradeModel2=$(sed -n 3p $img_ver)
	upgradeModel3=$(sed -n 4p $img_ver)
	echo "will upgrade mode : $upgradeModel1 or $upgradeModel2 or $upgradeModel3"

	if cat /var/config/lastgood_hs.xml | grep $upgradeModel1; then 
		echo "model is the same........"
	elif cat /var/config/lastgood_hs.xml | grep $upgradeModel2; then
		echo "model is the same........"
	elif cat /var/config/lastgood_hs.xml | grep $upgradeModel3; then
		echo "model is the same........"
	else
		echo "model is not the same........"
		
		exit 1
	fi
	###################IMG SDK VERSION VERTIFICATION####################
	echo IMG SDK VERSION VERTIFICATION
	img_sdk_version=$(sed -n 1p $img_ver | awk -F"-" '{print $1}')
	onu_sdk_version=$(cat /etc/version | awk -F"-" '{print $1}')
	echo $img_sdk_version
	echo $onu_sdk_version
	for i in 1 2 3 4 5 6 7 8 9 10;
	do
	 img_sdk_version_tail=0$(echo $img_sdk_version | awk -F "." "{print $"$i"}")
	 if [ $img_sdk_version_tail == 0 ]
	  then
	  	img_sdk_version_num=$i
	  break
	 fi 
	done  
	echo $i
	for i in 1 2 3 4 5 6 7 8 9 10;
	do
	 onu_sdk_version_tail=0$(echo $onu_sdk_version | awk -F "." "{print $"$i"}")
	 if [ $onu_sdk_version_tail == 0 ]
	  then
	  	onu_sdk_version_num=$i
	  break
	 fi 
	done  
	echo $i
	if [ $onu_sdk_version_num != $img_sdk_version_num ]
	then
		echo "sdk version is not the same........"
		exit 1
	fi
	echo "sdk version is the same........"
elif 
	cat /var/config/lastgood.xml | grep 'Value Name="ONUTYPE_CHECK_SWITCH" Value="0"'
then
	echo "Switch ONUTYPE_CHECK_SWITCH is 0 , skip the onu check........"
else
	echo "Can not find switch ONUTYPE_CHECK_SWITCH , skip the onu check........"
fi
#end add

# Stop this script upon any error
# set -e

echo "Updating image $1 with file $2"

# Find out kernel/rootfs mtd partition according to image destination
# Modified only one partion for each image 20160318.
#k_mtd="/dev/"`cat /proc/mtd | grep k"$1" | sed 's/:.*$//g'`
#r_mtd="/dev/"`cat /proc/mtd | grep r"$1" | sed 's/:.*$//g'`
k_mtd="/dev/"`cat /proc/mtd | grep "$kname" | sed 's/:.*$//g'`
r_mtd="/dev/"`cat /proc/mtd | grep "$rname" | sed 's/:.*$//g'`

#Ended by Samson.
echo "kernel image is located at $k_mtd"
echo "rootfs image is located at $r_mtd"

# Extract fwu_ver
tar -xf $2 $img_ver -O | md5sum | sed 's/-/'$img_ver'/g' > $md5_rt_result
# Check integrity
grep $img_ver $md5_cmp > $md5_tmp
diff $md5_rt_result $md5_tmp

if [ $? != 0 ]; then
    echo "$img_ver""md5_sum inconsistent, aborted image updating !"
    exit 1
fi

# Extract fwu.sh
tar -xf $2 $img_sh -O | md5sum | sed 's/-/'$img_sh'/g' > $md5_rt_result
# Check integrity
grep $img_sh $md5_cmp > $md5_tmp
diff $md5_rt_result $md5_tmp

if [ $? != 0 ]; then
    echo "$img_sh""md5_sum inconsistent, aborted image updating !"
    exit 1
fi

# Extract kernel image
tar -xf $2 $k_img -O | md5sum | sed 's/-/'$k_img'/g' > $md5_rt_result
# Check integrity
grep $k_img $md5_cmp > $md5_tmp
diff $md5_rt_result $md5_tmp

if [ $? != 0 ]; then
    echo "$k_img""md5_sum inconsistent, aborted image updating !"
    exit 1
fi

# Extract rootfs image
tar -xf $2 $r_img -O | md5sum | sed 's/-/'$r_img'/g' > $md5_rt_result
# Check integrity
grep $r_img $md5_cmp > $md5_tmp
diff $md5_rt_result $md5_tmp

if [ $? != 0 ]; then
    # rm $r_img
    echo "$r_img""md5_sum inconsistent, aborted image updating !"
    exit 1
fi
echo "Integrity of $k_img & $r_img is okay, start updating"

###################uImage####################
#tar 16ֵ
len1=$(tar -vtf $2 $k_img | awk '{print $3}')
echo $len1
new1=$(printf "%x" $len1)
echo $k_img size is $new1
#mtd 16ֵ
value=$(cat /proc/mtd | grep $kname | awk '{print $2}')
echo $value
value_1=$(echo 0x$value)
value1=$(printf %x $value_1)
echo mtd size is $value1

#ȽϽ
if [ $value1 \> $new1 ]
then echo $k_img space is enough for update!
else echo $k_img space is not enough for update!
exit 1
fi

###################rootfs###################
#tar 16ֵ
len2=$(tar -vtf $2 $r_img | awk '{print $3}')
echo $len2
new2=$(printf "%x" $len2)
echo $r_img size is $new2
#mtd 16ֵ
value=$(cat /proc/mtd | grep $rname | awk '{print $2}')
echo $value
value_2=$(echo 0x$value)
value2=$(printf %x $value_2)
echo mtd size is $value2

#ȽϽ
if [ $value2 \> $new2 ]
then echo $r_img space is enough for update!
else echo $r_img space is not enough for update! 
exit 1
fi

echo 0 > /proc/watchdog/watchdog_flag	

#add by abner for bug#0000721 on 2017-11-14
tar -xf $2 $img_ver 
tempversion=$(sed -n 1p $img_ver)
version=${tempversion:0:8}

echo "version: $version"
nv setenv sw_version0 $version
nv setenv sw_version1 $version
#nv setenv sw_version0 $(sed -n 1p $img_ver)
#nv setenv sw_version1 $(sed -n 1p $img_ver)

# Erase kernel partition 
flash_eraseall $k_mtd
# Write kernel partition
echo "Writing $k_img to $k_mtd"
tar -xf $2 $k_img -O > $k_mtd

# Erase rootfs partition 
flash_eraseall $r_mtd
# Write rootfs partition
echo "Writing $r_img to $r_mtd"
tar -xf $2 $r_img -O > $r_mtd


#add by abner for bug#0000586 on 2017-09-25
echo 1 > /proc/watchdog/watchdog_flag	
echo 80 > /proc/watchdog/kick_wdg_time			#need >62


# Write image version information 
# tar -xf $2 $img_ver 
# nv setenv sw_version"$1" "`cat $img_ver`"

# Clean up temporary files
rm -f $md5_cmp $md5_tmp $md5_rt_result $img_ver $2

# Post processing (for future extension consideration)

echo "Successfully updated image $1!!"

